* Codes to estimate the Random Parameter Model following Apesteguia & Ballester:
* MONOTONE STOCHASTIC CHOICE MODELS: THE CASE OF RISK AND TIME PREFERENCES
* Journal of Political Economy, Forthcoming
*
* Written by Jose Apesteguia, Miguel A. Ballester and Angelo Gutierrez-Daza
* November 2016
*
* Tested on Stata 14

* Initialize
clear all
cls
set more off

* Change the following line to match the directory with the files
cd "C:\Users\agutieda\Desktop\MSCM_CODES_WEBSITE\CodesRPM\STATA"

/* ****************************** The Data ****************************************************** */
** For illustrative purposes, we use data from Andersen et. al.(2008) field experiment on a 
** representative sample of the adult Danish population comprised of 253 subjects. In the experiment,
** there were four different risk-aversion choice tasks in the style of the multiple-price lists of
** Holt and Laury (2002). Each task comprised of ten pairs of nested gambles. For every pair of 
** gambles, subjects could either choose one of the gambles, or express indifference between the two. 
** In the latter case, they were told that the experimenter would settle indifferences by tossing a
** fair coin. We pool all observations from all individuals to get a sample of approximately 
** 8000 observations.

qui do prepare_data.do
use TestData.dta, clear


/* * Computing the risk aversion level that equalizes the expected utility of the two lotteries * */
** Estimation of the RPM model requires computation of the risk aversion level that makes an
** individual indifferent between the two lotteries considered in each observation (omega, following
** the notation in our paper). For this purpose, we have written a routine in Mata that goes through 
** each observation and computes omega by solving numerically the non-linear equation implied by the
** equalization of the expected utilities of each lottery. The code is found in the file 
** "compute_omega.m" and makes use of Mata language. The program is flexible, fast and easy to adapt
** to other datasets or utility functions

qui do compute_omega.do

* Notice that a new variable "OMEGA" has been added to the database


/* ************************ Estimate the RPM Model (Logit) ************************************** */

* First, we have to define the log-likelihood functions used for estimation 
* (check the define_loglikelihood.do file to see how this is done)
qui do define_loglikelihood.do

* Now, use the Stata's command "ml" to estimate the model parameter's by maximizing the
* log-likelihood function using the data for all individuals

* Some reminders:
* - The "lf" part tells Stata we are estimating a log-likelihood without providing numerical derivatives
* - "loglike_RPM_logit" is the name of the log-likelihood defined in the define_loglikelihood.do file
* - Each parenthesis specifies a parameter to be estimated in the mode (r: )(LN_lambda: )(kapp: )
* - The variables declared after ":" give arguments to the log-likelihood function. They must be 
*   declared in the same order implied when the function was defined.
* - We can add additional explanatory variables for each parameter like income and age by including 
*   them after the sign "="
* - We pool all responses to estimate the parameters but cluster the standard errors by individual 
*   with the command cluster(id)
* - "technique(nr)" sets the numerical method used to maximize the log-likelihood. In this case, the
*    Newton-Rapson method
* - Finally, "maximize" lets Stata know that we want to maximize, not minimize, the log-likelihood
ml model lf loglike_RPM_logit (r: CHOICE OMEGA = ) (LN_lambda: ) (kapp: ) , ///
	robust cluster(id) technique(nr) maximize

* Now display the results of the estimation
ml display

* Since we estimated the log of lambda and not it's level, we can use the delta method to recover
* the point estimate and std. dev. of lambda using the following command
nlcom (lambda: exp([LN_lambda]_b[_cons]))

* Alternatively, we can specify better initial values if the maximization procedure is having problems.
* To do so, we proceed as follows

* 1) Specify the model
ml model lf loglike_RPM_logit (r: CHOICE OMEGA = ) (LN_lambda: ) (kapp: ) , ///
	robust cluster(id) technique(nr)
	
* 2) Specify the initial values for the maximization of the log-likelihood
ml init  r:_cons=0.751  /LN_lambda:_cons=0.914 /kapp:_cons=0.051

* 3) Ask Stata to estimate the model
ml maximize

* 4) Use delta method to recover estimate of lambda from its log-estimate
nlcom (lambda: exp([LN_lambda]_b[_cons]))


/* ********************** Estimate the RPM Model using Probit *********************************** */
* Same as above but using the loglikelihood for the Probit model instead
* (also defined in the define_loglikelihood.do file)
ml model lf loglike_RPM_probit (r: CHOICE OMEGA = ) (LN_lambda: ) (kapp: ) , ///
	robust cluster(id) technique(nr) maximize
ml display
nlcom (lambda: exp([LN_lambda]_b[_cons]))

/* ******************************** Adding regressors ******************************************* */
* A nice thing about Stata is that it allows to easily add explanatory variables for the risk 
* aversion coefficient. Here, we show how to estimate the RPM model as a function of a constant and
* the dummy "old" in the original dataset
ml model lf loglike_RPM_logit (r: CHOICE OMEGA = OLD) (LN_lambda: ) (kapp: ) , ///
	robust cluster(id) technique(nr) maximize
ml display
nlcom (lambda: exp([LN_lambda]_b[_cons]))
