* This file computes numerically the value of risk aversion that equalizes the expected value 
* of the lottery described in each observation, assuming a CRRA utility function

* Open Mata
mata:

// First, define a function that takes as argument a risk aversion level. 
// For the two lotteries with the same probability but different payoffs from the dataset,
// the and returns the difference between the expected value of both lotteries

/* *******************************************************************************  */
void function EU_DIFF(real scalar r, real scalar eu_diff){
// The function receives as input a vector of arguments (the risk aversion level)
// And returns another scalar (the difference between expected utilities)

// Takes as external input the lotteries under consideration
external p    // Probability of outcome 1 in each lottery  
external x1   // Money rewarded in outcome 1 of lottery X
external x2   // Money rewarded in outcome 2 of lottery X
external y1   // Money rewarded in outcome 1 of lottery Y
external y2   // Money rewarded in outcome 2 of lottery Y 
external scale // Scale factor of the payments             

// Compute the utility generated by each outcome of both lotteries
uX1 = ( (scale*x1)^(1-r) ) / ( 1-r )
uX2 = ( (scale*x2)^(1-r) ) / ( 1-r )
uY1 = ( (scale*y1)^(1-r) ) / ( 1-r )
uY2 = ( (scale*y2)^(1-r) ) / ( 1-r )

// Compute the Expected Utility of each lottery
euX = p*uX1 + (1-p)*uX2 
euY = p*uY1 + (1-p)*uY2

// Compute the difference between the expected utility of both lotteries
eu_diff = euX-euY

}
/* *******************************************************************************  */

// Next, we create a new variable in the database where the variable omega of each individual will be stored
omega_idx_stata = st_addvar("float","OMEGA")

// And get the number of observations in the dataset 
n_obs = st_nobs() 

// Now we create column vectors containing the payoffs and probabilities of the lotteries used in each observation

// Initialization
X1_vec   = 0 
X2_vec   = 0
Y1_vec   = 0
Y2_vec   = 0
p_vec     = 0

// Scale factor for the payoffs
// This doesn't affect the final estimates both makes easier to solve the equations numerically in Stata
scale = 1/1000

// Get the variables from the Stata database
st_view(X1_vec    , ., "X1")
st_view(X2_vec    , ., "X2")
st_view(Y1_vec    , ., "Y1")
st_view(Y2_vec    , ., "Y2")
st_view(p_vec     , ., "P")

// Finally, we set a loop that iterates over each observation and computes the level 
// of risk aversion that makes the individual indifferent between both lotteries. 
// This is achieved by solving numerically the non-linear equation defined by function EU_DIFF above. 

// Before that, we set up the numerical solver

/* ******************************************************************************** */

// Set initial value for the solve
init_val = -2

// Set up the numerical solver
S_i = solvenl_init()

// Specify the function with the non-linear equation to be solved	
solvenl_init_evaluator(S_i, &EU_DIFF())	

// Specify that the problem is to find a solution to the equation
solvenl_init_type(S_i, "zero")

// Specify the numerical procedure to be used
//solvenl_init_technique(S_i, "broydenpowell")
solvenl_init_technique(S_i, "newtonraphson") 

// Specify the number of equations in the system to be solved
solvenl_init_numeq(S_i, 1)

// Specify the initial value
solvenl_init_startingvals(S_i, init_val)

// Do not show iterations of this process
solvenl_init_iter_log(S_i, "off")

// Tolerance: f(x(t))-f(x(t-1))
solvenl_init_conv_nearzero(S_i, 1e-10)

// Tolerance: x(t+1)-x(t)
solvenl_init_conv_iterchng(S_i, 1e-10)

// Tolerance: Maximum number of iterations
solvenl_init_conv_maxiter(S_i, 1000)

/* ******************************************************************************** */

// Compute omega
omega_vec = J(n_obs,1,.)
for (i=1; i<=n_obs; i++){

	// Define the corresponding lottery of observation "i"
	x1   = X1_vec[i]
	x2   = X2_vec[i]
	y1   = Y1_vec[i]
	y2   = Y2_vec[i]
	p     = p_vec[i]  

	// Finally, solve the non-linear equation
	omega_vec[i] = solvenl_solve(S_i)
	
}

// Store the critical value found in the dataset
st_store(.,omega_idx_stata,omega_vec)

// Close Mata
end

